/***************************************************************************
 *   Copyright (C) 2005 by Pawel Nawrocki                                  *
 *   pnawrocki@interia.pl                                                  *
 *                                                                         *
 *   This program is free software; you can redistribute it and/or modify  *
 *   it under the terms of the GNU General Public License as published by  *
 *   the Free Software Foundation; either version 2 of the License, or     *
 *   (at your option) any later version.                                   *
 *                                                                         *
 *   This program is distributed in the hope that it will be useful,       *
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of        *
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the         *
 *   GNU General Public License for more details.                          *
 *                                                                         *
 *   You should have received a copy of the GNU General Public License     *
 *   along with this program; if not, write to the                         *
 *   Free Software Foundation, Inc.,                                       *
 *   59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.             *
 ***************************************************************************/

#include <iostream>
#include <qprocess.h>
#include <kmessagebox.h>

class WANetParams
{
public:
        QString iface;
        QString essid;
        //QString mode;
        QString channel;
        QString ap;
        bool wep;
        QString wepMode;
        QString wepKey;

        bool dhcp;
        QString ip;
        QString netmask;
        QString broadcast;
        QString gateway;
        QString domain;
        QString dns1;
        QString dns2;

        bool hiddenEssid;

        QString netParamsString()
        {
                QStringList mNPS;
                mNPS << boolToString(hiddenEssid) << essid << ap << channel << boolToString(wep) << wepMode << wepKey << boolToString(dhcp) << ip << netmask << broadcast << gateway << domain << dns1 << dns2;
                return mNPS.join(",");
        }

        void loadNetParamsString( const QString & nps )
        {
                if (nps.section(",",0,0)=="true")
                        hiddenEssid=true;
                else
                        hiddenEssid=false;
                essid = nps.section(",",1,1);
                ap = nps.section(",",2,2);
                channel = nps.section(",",3,3);
                /*if (nps.section(",",4,4)=="true")
                	wep=true;
                else
                	wep=false;*/	// COMMENTED OUT because wep boolean is set from the list item
                wepMode = nps.section(",",5,5);
                wepKey = nps.section(",",6,6);
                if (nps.section(",",7,7)=="true")
                        dhcp=true;
                else
                        dhcp=false;
                ip = nps.section(",",8,8);
                netmask = nps.section(",",9,9);
                broadcast = nps.section(",",10,10);
                gateway = nps.section(",",11,11);
                domain = nps.section(",",12,12);
                dns1 = nps.section(",",13,13);
                dns2 = nps.section(",",14,14);
        }
private:
        QString boolToString( bool b )
        {
                QString result;
                if ( b )
                        result = "true";
                else
                        result = "false";
                return result;
        }
};

class WACommands
{
public:
        bool allFound;
        QStringList notFound;

        void init()
        {
                QStringList binDirs;
                binDirs << "/sbin" << "/usr/sbin" << "/usr/local/sbin" << "/bin" << "/usr/bin" << "/usr/local/bin";

                dhcp = getPath("dhcpcd", binDirs); //these 2 checks have to be first, so allFound flag is properly set.
                if (dhcp.isEmpty())
                        dhcp = getPath("dhclient", binDirs);
                if (!dhcp.isEmpty()) {
                        allFound=1;
                        dhcpClient = dhcp.section("/",-1,-1);
                        std::cout << "DHCP Client: " << dhcpClient << std::endl;
                }

                ifconfig = getPath("ifconfig", binDirs);
                iwconfig = getPath("iwconfig", binDirs);
                iwlist = getPath("iwlist", binDirs);
                route = getPath("route", binDirs);
                pidof = getPath("pidof", binDirs);

                if (!allFound)
                        std::cout << "Executable(s) not found:" << notFound.join(", ") << std::endl;
                else
                        std::cout << "All executables found." << std::endl;
        }

        QStringList cmd( const QString & action, const WANetParams & np )
        {
                QStringList mCmd;

                if (action=="ifup")
                        mCmd << ifconfig << np.iface << "up";
                else if (action=="ifdown")
                        mCmd << ifconfig << np.iface << "down";
                else if (action=="scan")
                        mCmd << iwlist << np.iface << "scan";
                else if (action=="disconnect")
                        mCmd << iwconfig << np.iface << "essid" << "\"\"" << "mode" << "managed" << "key" << "off";
                else if (action=="iwconfig") {
                        mCmd << iwconfig << np.iface << "essid" << np.essid << "mode" << "managed" << "channel" << np.channel << "key";
                        if (np.wep)
                                mCmd << np.wepMode << np.wepKey;
                        else
                                mCmd << "off";
                } else if (action=="iwconfig_ap") {
                        mCmd << iwconfig << np.iface << "ap" << np.ap;

                } else if (action=="ifconfig_dhcp") {
                        if (dhcp.find("dhcpcd") > -1)
                                mCmd << dhcp << "-nd" << "-t" << "25" << np.iface;
                        else
                                mCmd << dhcp << "-1" << "-q" << np.iface;
                } else if (action=="kill_dhcp") {
                        if (dhcpClient=="dhcpcd") {
                                if ( QFile(QString("/var/run/dhcpcd-"+np.iface+".pid")).exists() || QFile(QString("/etc/dhcpc/dhcpcd-"+np.iface+".pid")).exists() )
                                        mCmd << dhcp << "-k" << np.iface;
                        } else { //dhclient
                                if ( QFile("/var/run/dhclient.pid").exists() )
                                        mCmd << dhcp << "-r" << np.iface;
                        }
                } else if (action=="ifconfig_manual") {
                        mCmd << ifconfig << np.iface << np.ip;
                        if (!np.netmask.isEmpty())
                                mCmd << "netmask" << np.netmask;
                        if (!np.broadcast.isEmpty())
                                mCmd << "broadcast" << np.broadcast;
                } else if (action=="route") {
                        if (!np.gateway.isEmpty())
                                mCmd << route << "add" << "default" << "gw" << np.gateway;
                } else if (action=="ping_gw") {
                        if (!np.gateway.isEmpty())
                                mCmd << "ping" << "-c" << "1" << "-I" << np.iface << np.gateway;
                } else if (action=="ping_internet") {
                                mCmd << "ping" << "-c" << "1" << "-I" << np.iface << "www.google.com";
                } else
                        std::cout << "Unknown action: " << action << std::endl;

                if (!mCmd.isEmpty()) //mCmd = QStringList();
                	std::cout << action << ": " << mCmd.join(" ") << std::endl;
                return mCmd;
        }

private:
        QString ifconfig;
        QString iwconfig;
        QString iwlist;
        QString dhcp;
        QString dhcpClient;
        QString route;
        QString pidof;

        QString getPath(QString file, QStringList dirs)
        {
                QString s;
                for ( QStringList::Iterator it = dirs.begin(); it != dirs.end(); it++ ) {
                        if (QFile( QString(*it+"/"+file) ).exists()) {
                                s = QString(*it+"/"+file);
                                break;
                        }
                }
                if (s.isEmpty()) {
                        allFound = 0;
                        notFound << file;
                }
                return s;
        }
};
